@@ -6,16 +6,45 @@ use warnings;
 use Test::More qw();
 use Scalar::Util qw(blessed);
 use Types::TypeTiny qw(to_TypeTiny);
-use base qw(Exporter::TypeTiny);
+
+require Exporter::Tiny;
+our @ISA = 'Exporter::Tiny';
 
 BEGIN {
 	*EXTENDED_TESTING = $ENV{EXTENDED_TESTING} ? sub(){!!1} : sub(){!!0};
 };
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.022';
+our $VERSION   = '1.000004';
 our @EXPORT    = qw( should_pass should_fail ok_subtype );
-our @EXPORT_OK = qw( EXTENDED_TESTING );
+our @EXPORT_OK = qw( EXTENDED_TESTING matchfor );
+
+sub matchfor
+{
+	my @matchers = @_;
+	bless \@matchers, do {
+		package #
+		Test::TypeTiny::Internal::MATCHFOR;
+		use overload
+			q[==] => 'match',
+			q[eq] => 'match',
+			q[""] => 'to_string',
+			fallback => 1;
+		sub to_string {
+			$_[0][0]
+		}
+		sub match {
+			my ($self, $e) = @_;
+			my $does = Scalar::Util::blessed($e) ? ($e->can('DOES') || $e->can('isa')) : undef;
+			for my $s (@$self) {
+				return 1 if  ref($s) && $e =~ $s;
+				return 1 if !ref($s) && $does && $e->$does($s);
+			}
+			return;
+		}
+		__PACKAGE__;
+	};
+}
 
 sub _mk_message
 {
@@ -114,12 +143,15 @@ Test::TypeTiny - useful functions for testing the efficacy of type constraints
 
 =head1 SYNOPSIS
 
+=for test_synopsis
+BEGIN { die "SKIP: uses a module that doesn't exist as an example" };
+
    use strict;
    use warnings;
    use Test::More;
    use Test::TypeTiny;
    
-   use Types::Mine qw(Integer);
+   use Types::Mine qw(Integer Number);
    
    should_pass(1, Integer);
    should_pass(-1, Integer);
@@ -130,6 +162,11 @@ Test::TypeTiny - useful functions for testing the efficacy of type constraints
    
    done_testing;
 
+=head1 STATUS
+
+This module is covered by the
+L<Type-Tiny stability policy|Type::Tiny::Manual::Policies/"STABILITY">.
+
 =head1 DESCRIPTION
 
 L<Test::TypeTiny> provides a few handy functions for testing type constraints.
@@ -158,6 +195,11 @@ Test that passes iff all C<< @subtypes >> are subtypes of C<< $type >>.
 
 Exportable boolean constant.
 
+=item C<< matchfor(@things) >>
+
+Assistant for matching exceptions. Not exported by default.
+See also L<Test::Fatal::matchfor>.
+
 =back
 
 =head1 ENVIRONMENT
@@ -188,7 +230,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013 by Toby Inkster.
+This software is copyright (c) 2013-2014 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
