@@ -6,14 +6,14 @@ use warnings;
 
 BEGIN {
 	$Type::Tiny::Duck::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Tiny::Duck::VERSION   = '0.044';
+	$Type::Tiny::Duck::VERSION   = '1.000004';
 }
 
 use Scalar::Util qw< blessed >;
 
 sub _croak ($;@) { require Error::TypeTiny; goto \&Error::TypeTiny::croak }
 
-require Type::Tiny;
+use Type::Tiny ();
 our @ISA = 'Type::Tiny';
 
 sub new {
@@ -27,6 +27,19 @@ sub new {
 	
 	$opts{methods} = [$opts{methods}] unless ref $opts{methods};
 	
+	if (Type::Tiny::_USE_XS)
+	{
+		my $methods = join ",", sort(@{$opts{methods}});
+		my $xsub    = Type::Tiny::XS::get_coderef_for("HasMethods[$methods]");
+		$opts{compiled_type_constraint} = $xsub if $xsub;
+	}
+	elsif (Type::Tiny::_USE_MOUSE)
+	{
+		require Mouse::Util::TypeConstraints;
+		my $maker = "Mouse::Util::TypeConstraints"->can("generate_can_predicate_for");
+		$opts{compiled_type_constraint} = $maker->($opts{methods}) if $maker;
+	}
+	
 	return $proto->SUPER::new(%opts);
 }
 
@@ -46,6 +59,14 @@ sub _build_inlined
 {
 	my $self = shift;
 	my @methods = @{$self->methods};
+	
+	if (Type::Tiny::_USE_XS)
+	{
+		my $methods = join ",", sort(@{$self->methods});
+		my $xsub    = Type::Tiny::XS::get_subname_for("HasMethods[$methods]");
+		return sub { my $var = $_[1]; "$xsub\($var\)" } if $xsub;
+	}
+	
 	sub {
 		my $var = $_[1];
 		local $" = q{ };
