@@ -6,9 +6,10 @@ use warnings;
 
 BEGIN {
 	$Types::Standard::HashRef::AUTHORITY = 'cpan:TOBYINK';
-	$Types::Standard::HashRef::VERSION   = '0.044';
+	$Types::Standard::HashRef::VERSION   = '1.000004';
 }
 
+use Type::Tiny ();
 use Types::Standard ();
 use Types::TypeTiny ();
 
@@ -24,17 +25,44 @@ sub __constraint_generator
 	Types::TypeTiny::TypeTiny->check($param)
 		or _croak("Parameter to HashRef[`a] expected to be a type constraint; got $param");
 	
-	return sub
+	my $param_compiled_check = $param->compiled_check;
+	my $xsub;
+	if (Type::Tiny::_USE_XS)
 	{
-		my $hash = shift;
-		$param->check($_) || return for values %$hash;
-		return !!1;
-	};
+		my $paramname = Type::Tiny::XS::is_known($param_compiled_check);
+		$xsub = Type::Tiny::XS::get_coderef_for("HashRef[$paramname]")
+			if $paramname;
+	}
+	elsif (Type::Tiny::_USE_MOUSE and $param->_has_xsub)
+	{
+		require Mouse::Util::TypeConstraints;
+		my $maker = "Mouse::Util::TypeConstraints"->can("_parameterize_HashRef_for");
+		$xsub = $maker->($param) if $maker;
+	}
+	
+	return (
+		sub
+		{
+			my $hash = shift;
+			$param->check($_) || return for values %$hash;
+			return !!1;
+		},
+		$xsub,
+	);
 }
 
 sub __inline_generator
 {
 	my $param = shift;
+	
+	my $compiled = $param->compiled_check;
+	if (Type::Tiny::_USE_XS)
+	{
+		my $paramname = Type::Tiny::XS::is_known($compiled);
+		my $xsubname  = Type::Tiny::XS::get_subname_for("HashRef[$paramname]");
+		return sub { "$xsubname\($_[1]\)" } if $xsubname;
+	}
+	
 	return unless $param->can_be_inlined;
 	my $param_check = $param->inline_check('$i');
 	return sub {
@@ -65,7 +93,8 @@ sub __deep_explanation
 		];
 	}
 	
-	return;
+	# This should never happen...
+	return;  # uncoverable statement
 }
 
 sub __coercion_generator
