@@ -4,11 +4,11 @@ use strict;
 use warnings;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.022';
+our $VERSION   = '1.000004';
 
-use Scalar::Util qw< blessed >;
+use Scalar::Util qw< blessed refaddr weaken >;
 
-our @EXPORT_OK = qw( CodeLike StringLike TypeTiny HashLike ArrayLike to_TypeTiny );
+our @EXPORT_OK = ( __PACKAGE__->type_names, qw/to_TypeTiny/ );
 
 my %cache;
 
@@ -16,11 +16,14 @@ sub import
 {
 	# do the shuffle!
 	no warnings "redefine";
-	our @ISA = qw( Exporter::TypeTiny );
-	require Exporter::TypeTiny;
-	my $next = \&Exporter::TypeTiny::import;
+	our @ISA = qw( Exporter::Tiny );
+	require Exporter::Tiny;
+	my $next = \&Exporter::Tiny::import;
 	*import = $next;
-	goto $next;
+	my $class = shift;
+	my $opts  = { ref($_[0]) ? %{+shift} : () };
+	$opts->{into} ||= scalar(caller);
+	return $class->$next($opts, @_);
 }
 
 sub meta
@@ -28,13 +31,42 @@ sub meta
 	return $_[0];
 }
 
+sub type_names
+{
+	qw( CodeLike StringLike TypeTiny HashLike ArrayLike );
+}
+
+sub has_type
+{
+	my %has = map +($_ => 1), shift->type_names;
+	!!$has{ $_[0] };
+}
+
 sub get_type
 {
 	my $self = shift;
-	my $func = $self->can(@_) or return;
-	my $type = $func->();
-	return $type if blessed($type) && $type->isa("Type::Tiny");
-	return;
+	return unless $self->has_type(@_);
+	no strict qw(refs);
+	&{$_[0]}();
+}
+
+sub coercion_names
+{
+	qw();
+}
+
+sub has_coercion
+{
+	my %has = map +($_ => 1), shift->coercion_names;
+	!!$has{ $_[0] };
+}
+
+sub get_coercion
+{
+	my $self = shift;
+	return unless $self->has_coercion(@_);
+	no strict qw(refs);
+	&{$_[0]}();  # uncoverable statement
 }
 
 sub StringLike ()
@@ -42,8 +74,8 @@ sub StringLike ()
 	require Type::Tiny;
 	$cache{StringLike} ||= "Type::Tiny"->new(
 		name       => "StringLike",
-		constraint => sub {    !ref($_   ) or Scalar::Util::blessed($_   ) && overload::Method($_   , q[""])  },
-		inlined    => sub { qq/!ref($_[1]) or Scalar::Util::blessed($_[1]) && overload::Method($_[1], q[""])/ },
+		constraint => sub {    defined($_   ) && !ref($_   ) or Scalar::Util::blessed($_   ) && overload::Method($_   , q[""])  },
+		inlined    => sub { qq/defined($_[1]) && !ref($_[1]) or Scalar::Util::blessed($_[1]) && overload::Method($_[1], q[""])/ },
 		library    => __PACKAGE__,
 	);
 }
@@ -92,24 +124,28 @@ sub TypeTiny ()
 	);
 }
 
+my %ttt_cache;
+
 sub to_TypeTiny
 {
 	my $t = $_[0];
 	
-	return $t unless ref $t;
-	return $t if ref($t) =~ /^Type::Tiny\b/;
+	return $t unless (my $ref = ref $t);
+	return $t if $ref =~ /^Type::Tiny\b/;
+	
+	return $ttt_cache{ refaddr($t) } if $ttt_cache{ refaddr($t) };
 	
 	if (my $class = blessed $t)
 	{
-		return $t                           if $class->isa("Type::Tiny");
-		goto \&_TypeTinyFromMoose           if $class->isa("Moose::Meta::TypeConstraint");
-		goto \&_TypeTinyFromMoose           if $class->isa("MooseX::Types::TypeDecorator");
-		goto \&_TypeTinyFromValidationClass if $class->isa("Validation::Class::Simple");
-		goto \&_TypeTinyFromValidationClass if $class->isa("Validation::Class");
-		goto \&_TypeTinyFromGeneric         if $t->can("check") && $t->can("get_message"); # i.e. Type::API::Constraint
+		return $t                               if $class->isa("Type::Tiny");
+		return _TypeTinyFromMoose($t)           if $class->isa("Moose::Meta::TypeConstraint");
+		return _TypeTinyFromMoose($t)           if $class->isa("MooseX::Types::TypeDecorator");
+		return _TypeTinyFromValidationClass($t) if $class->isa("Validation::Class::Simple");
+		return _TypeTinyFromValidationClass($t) if $class->isa("Validation::Class");
+		return _TypeTinyFromGeneric($t)         if $t->can("check") && $t->can("get_message"); # i.e. Type::API::Constraint
 	}
 	
-	goto \&_TypeTinyFromCodeRef if ref($t) eq q(CODE);
+	return _TypeTinyFromCodeRef($t) if $ref eq q(CODE);
 	
 	$t;
 }
@@ -123,7 +159,8 @@ sub _TypeTinyFromMoose
 		return $t->{"Types::TypeTiny::to_TypeTiny"};
 	}
 	
-	if ($t->name ne '__ANON__') {
+	if ($t->name ne '__ANON__')
+	{
 		require Types::Standard;
 		my $ts = 'Types::Standard'->get_type($t->name);
 		return $ts if $ts->{_is_core};
@@ -138,7 +175,19 @@ sub _TypeTinyFromMoose
 	$opts{moose_type}   = $t;
 	
 	require Type::Tiny;
-	return "Type::Tiny"->new(%opts);
+	my $new = 'Type::Tiny'->new(%opts);
+	$ttt_cache{ refaddr($t) } = $new;
+	weaken($ttt_cache{ refaddr($t) });
+	
+	$new->{coercion} = do {
+		require Type::Coercion::FromMoose;
+		'Type::Coercion::FromMoose'->new(
+			type_constraint => $new,
+			moose_coercion  => $t->coercion,
+		);
+	} if $t->has_coercion;
+	
+	return $new;
 }
 
 sub _TypeTinyFromValidationClass
@@ -190,6 +239,7 @@ sub _TypeTinyFromValidationClass
 		};
 	}
 	
+	require Type::Tiny;
 	my $new = "Type::Tiny"->new(%opts);
 	
 	$new->coercion->add_type_coercions(
@@ -204,6 +254,8 @@ sub _TypeTinyFromValidationClass
 		},
 	);
 	
+	$ttt_cache{ refaddr($t) } = $new;
+	weaken($ttt_cache{ refaddr($t) });
 	return $new;
 }
 
@@ -212,7 +264,6 @@ sub _TypeTinyFromGeneric
 	my $t = $_[0];
 	
 	# XXX - handle inlining??
-	# XXX - handle display_name????
 	
 	my %opts = (
 		constraint => sub { $t->check(@_ ? @_ : $_) },
@@ -225,15 +276,18 @@ sub _TypeTinyFromGeneric
 		if $t->can("has_coercion") && $t->has_coercion && $t->can("coerce");
 	
 	require Type::Tiny;
-	return "Type::Tiny"->new(%opts);
+	my $new = "Type::Tiny"->new(%opts);
+	$ttt_cache{ refaddr($t) } = $new;
+	weaken($ttt_cache{ refaddr($t) });
+	return $new;
 }
 
+my $QFS;
 sub _TypeTinyFromCodeRef
 {
 	my $t = $_[0];
 	
-	require Type::Tiny;
-	return "Type::Tiny"->new(
+	my %opts = (
 		constraint => sub {
 			return !!eval { $t->($_) };
 		},
@@ -243,6 +297,31 @@ sub _TypeTinyFromCodeRef
 			return sprintf('%s did not pass type constraint', Type::Tiny::_dd($_));
 		},
 	);
+	
+	if ($QFS ||= "Sub::Quote"->can("quoted_from_sub"))
+	{
+		my (undef, $perlstring, $captures) = @{ $QFS->($t) || [] };
+		if ($perlstring)
+		{
+			$perlstring = "!!eval{ $perlstring }";
+			$opts{inlined} = sub
+			{
+				my $var = $_[1];
+				Sub::Quote::inlinify(
+					$perlstring,
+					$var,
+					$var eq q($_) ? '' : "local \$_ = $var;",
+					1,
+				);
+			} if $perlstring && !$captures;
+		}
+	}
+	
+	require Type::Tiny;
+	my $new = "Type::Tiny"->new(%opts);
+	$ttt_cache{ refaddr($t) } = $new;
+	weaken($ttt_cache{ refaddr($t) });
+	return $new;
 }
 
 1;
@@ -253,16 +332,24 @@ __END__
 
 =encoding utf-8
 
+=for stopwords arrayfication hashification
+
 =head1 NAME
 
 Types::TypeTiny - type constraints used internally by Type::Tiny
 
+=head1 STATUS
+
+This module is covered by the
+L<Type-Tiny stability policy|Type::Tiny::Manual::Policies/"STABILITY">.
+
 =head1 DESCRIPTION
 
 Dogfooding.
 
-This isn't a real Type::Library-based type library; that would involve too
-much circularity. But it exports some type constraint "constants":
+This isn't a real Type::Library-based type library; that would involve
+too much circularity. But it exports some type constraints which, while
+designed for use within Type::Tiny, may be more generally useful.
 
 =head2 Types
 
@@ -270,14 +357,24 @@ much circularity. But it exports some type constraint "constants":
 
 =item C<< StringLike >>
 
+Accepts strings and objects overloading stringification.
+
 =item C<< HashLike >>
 
+Accepts hashrefs and objects overloading hashification.
+
 =item C<< ArrayLike >>
 
+Accepts arrayrefs and objects overloading arrayfication.
+
 =item C<< CodeLike >>
 
+Accepts coderefs and objects overloading codification.
+
 =item C<< TypeTiny >>
 
+Accepts blessed L<Type::Tiny> objects.
+
 =back
 
 =head2 Coercion Functions
@@ -315,8 +412,18 @@ works, for rough compatibility with a real L<Type::Library> type library.
 
 =item C<< meta >>
 
+=item C<< type_names >>
+
 =item C<< get_type($name) >>
 
+=item C<< has_type($name) >>
+
+=item C<< coercion_names >>
+
+=item C<< get_coercion($name) >>
+
+=item C<< has_coercion($name) >>
+
 =back
 
 =head1 BUGS
@@ -334,7 +441,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013 by Toby Inkster.
+This software is copyright (c) 2013-2014 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
